//
//  ZRAsync.m
//  ZeroEdit
//
//  Created by ampatspell on 6/4/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "ZRAsync.h"


NSString *const ZRAsyncDidBecomeBusyNotification = @"ZRAsyncDidBecomeBusyNotification";
NSString *const ZRAsyncDidBecomeIdleNotification = @"ZRAsyncDidBecomeIdleNotification";

NSString *const ZRAsyncErrorDomain = @"ZRAsyncErrorDomain";
NSInteger const ZRAsyncRequestFailedErrorCode = 10;

ZRAsyncRequestID const ZRNilAsyncRequestID = nil;

@interface ZRAsync ()
@property(nonatomic, readwrite, retain) NSMutableDictionary *operations;

- (void)_addOperation:(ZRRPCOperation *)operation forRequestId:(ZRAsyncRequestID)requestId;
- (ZRAsyncRequestID)_requestIdForOperation:(ZRRPCOperation *)operation;
- (ZRRPCOperation *)_operationForRequestId:(ZRAsyncRequestID)requestId;
- (void)_operationRequestDidFinish:(ZRRPCOperation *)operation;

- (NSUInteger)_countOfOperations;
- (NSArray *)_allRequestIds;
@end


@implementation ZRAsync
@synthesize operations, delegate;

- (id)init
{
  return [self initWithDelegate:nil];
}

- (id)initWithDelegate:(id<ZRAsyncDelegate>)aDelegate
{
  if(self = [super init]) {
    delegate = aDelegate;
    operations = [[NSMutableDictionary alloc] init];
  }
  return self;  
}

- (void)dealloc
{
  [self cancelAll];
  self.operations = nil;
  [super dealloc];
}

#pragma mark -

- (ZRAsyncRequestID)executeWithURL:(NSURL *)url
                        parameters:(NSDictionary *)parameters 
                          userInfo:(id)userInfo
{
  CFUUIDRef uuidRef = CFUUIDCreate(NULL);
  NSString *requestId = [(NSString *)CFUUIDCreateString(NULL, uuidRef) autorelease];
  CFRelease(uuidRef);
  
#ifdef ZR_ASYNC_DEBUG
  NSLog(@"ZRAsync EXECUTE\nRequest ID: %@\nURL: %@\n%@\n\n", requestId, url, parameters);
#endif
  
  ZRRPCOperation *operation = [[ZRRPCOperation alloc] init];
  operation.url = url;
  operation.parameters = parameters;
  operation.delegate = self;
  operation.userInfo = userInfo;
  
  [self _addOperation:operation forRequestId:requestId];
  [operation release];
  
  return requestId;
}

- (void)cancel:(ZRAsyncRequestID)requestId
{
  ZRRPCOperation *operation = [self _operationForRequestId:requestId];
  if(operation) {    
    [operation cancel];
    [self _operationRequestDidFinish:operation];
  }
}

- (void)cancelAll
{
  for(ZRAsyncRequestID requestId in [self _allRequestIds]) {
    [self cancel:requestId];
  }
}

#pragma mark -

- (void)_addOperation:(ZRRPCOperation *)operation forRequestId:(ZRAsyncRequestID)requestId
{
  [self.operations setObject:operation forKey:requestId];
  [operation start];
  
  if([self _countOfOperations] == 1) {
    [[NSNotificationCenter defaultCenter] postNotificationName:ZRAsyncDidBecomeBusyNotification object:self];    
  }
}

- (NSUInteger)_countOfOperations
{
  return [self.operations count];
}

- (NSArray *)_allRequestIds
{
  return [[self.operations allKeys] copy];
}

- (ZRRPCOperation *)_operationForRequestId:(ZRAsyncRequestID)requestId
{
  return [self.operations objectForKey:requestId];
}

- (ZRAsyncRequestID)_requestIdForOperation:(ZRRPCOperation *)operation
{
  for(ZRAsyncRequestID requestId in self.operations) {
    if([self.operations objectForKey:requestId] == operation) {
      return requestId;
    }
  }
  return nil;
}

- (void)_operationRequestDidFinish:(ZRRPCOperation *)operation
{
  ZRAsyncRequestID requestId = [self _requestIdForOperation:operation];
  [self.operations removeObjectForKey:requestId];
  
  if([self _countOfOperations] == 0) {
    [[NSNotificationCenter defaultCenter] postNotificationName:ZRAsyncDidBecomeIdleNotification object:self];    
  }
}

- (void)RPCOperation:(ZRRPCOperation *)operation didSucceed:(id)json
{
#ifdef ZR_ASYNC_DEBUG
  NSLog(@"ZRAsync SUCCEEDED\nRequest ID: %@\n%@\n\n", [self _requestIdForOperation:operation], json);
#endif
  [delegate async:self operationDidSucceedWithResult:json userInfo:operation.userInfo];
  [self _operationRequestDidFinish:operation];
}

- (void)RPCOperation:(ZRRPCOperation *)operation didFailWithError:(NSError *)error
{
#ifdef ZR_ASYNC_DEBUG
  NSLog(@"ZRAsync FAILED\nRequest ID: %@\n%@\n\n", [self _requestIdForOperation:operation], error);
#endif
  [delegate async:self operationDidFailWithError:error userInfo:operation.userInfo];
  [self _operationRequestDidFinish:operation];
}

@end
