//
//  ZRAsyncDispatch.m
//  AsyncDispatch
//
//  Created by ampatspell on 7/23/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "ZRAsyncDispatch.h"
#import "ZROperation.h"
#import "ZRQuery.h"
#import "ZRModelConfiguration.h"


@implementation ZRAsyncDispatch
@synthesize url, delegate;

- (id)initWithURL:(NSURL *)anURL
{
  if(self = [super init]) {
    self.url = anURL;
    async = [[ZRAsync alloc] initWithDelegate:self];
  }
  return self;
}

- (void)dealloc
{
  [async cancelAll];
  
  self.url = nil;
  [async release], async = nil;
  delegate = nil;
  [super dealloc];
}

#pragma mark -

- (id)_arrayForClass:(Class)modelClass withQuery:(ZRQueryKind)kind parameters:(NSDictionary *)params state:(ZRModelState)state
{
  ZRModelConfiguration *configuration = [modelClass modelConfiguration];
  ZRQuery *query = [configuration queryForKind:kind];
  if(!query) {
    return nil;
  }
  
  Class arrayClass = [query arrayClass];
  id array = [[arrayClass alloc] initWithModelClass:modelClass query:query parameters:params state:state delegate:self];
  
  if([delegate respondsToSelector:@selector(asyncDispatch:didCreateArrayInstance:)]) {
    [delegate asyncDispatch:self didCreateModelArrayInstance:array];
  }
  
  return [array autorelease];
}


- (id)arrayForClass:(Class)modelClass withQuery:(ZRQueryKind)kind parameters:(NSDictionary *)params
{
  return [self _arrayForClass:modelClass withQuery:kind parameters:params state:ZRModelStateBlank | ZRModelStateIdle];
}

- (id)arrayForClass:(Class)modelClass withQuery:(ZRQueryKind)kind
{
  return [self arrayForClass:modelClass withQuery:kind parameters:nil];
}

#pragma mark -

- (ZRModel *)_modelForClass:(Class)modelClass withKey:(NSString *)key state:(ZRModelState)state
{
  id model = [[[modelClass alloc] initWithKey:key state:state delegate:self] autorelease];
  if([delegate respondsToSelector:@selector(asyncDispatch:didCreateModelInstance:)]) {
    [delegate asyncDispatch:self didCreateModelInstance:model];    
  }
  return model;
}

- (id)modelForClass:(Class)modelClass
{
  return [self _modelForClass:modelClass withKey:nil state:ZRModelStateNew | ZRModelStateIdle];
}

- (id)modelForClass:(Class)modelClass withKey:(NSString *)key
{
  return [self _modelForClass:modelClass withKey:key state:ZRModelStateBlank | ZRModelStateIdle];
}

- (NSDictionary *)_parametersForOperationParameters:(NSDictionary *)parameters
{
  if(![delegate respondsToSelector:@selector(asyncDispatchAdditionalParameters:)]) {
    return parameters;
  }
  
  NSDictionary *additionalParameters = [delegate asyncDispatchAdditionalParameters:self];
  if(!additionalParameters) {
    return parameters;
  }
  
  NSMutableDictionary *concatenatedParameters = [NSMutableDictionary dictionary];
  [concatenatedParameters addEntriesFromDictionary:additionalParameters];
  [concatenatedParameters addEntriesFromDictionary:parameters];
  return concatenatedParameters;
}

#pragma mark -

- (void)performOperation:(ZROperation *)operation
{
  NSString *operationAction = operation.action;
  if([operation.action hasPrefix:@"/"]) {
    operationAction = [operation.action substringFromIndex:1];
  }

  NSURL        *operationURL        = [url URLByAppendingPathComponent:operationAction];
  NSDictionary *operationParameters = [self _parametersForOperationParameters:operation.parameters];
  
  [async executeWithURL:operationURL parameters:operationParameters userInfo:operation];
}

- (BOOL)retryOperation:(ZROperation *)operation withError:(NSError *)error
{
  if([delegate asyncDispatch:self shouldRetryOperation:operation withError:error]) {
    [operation incrementRetryCount];
    [self performOperation:operation];
    return YES;
  }
  return NO;
}

- (id)createdModelForClass:(Class)modelClass fromDictionary:(NSDictionary *)dictionary
{
  ZRModel *model = [self _modelForClass:modelClass withKey:nil state:ZRModelStateCreated | ZRModelStateIdle];
  [model setDisableDirtyRegistration:YES];
  [model setModelPropertiesFromDictionary:dictionary];
  [model setDisableDirtyRegistration:NO];
  return model;
}

#pragma mark -

- (void)async:(ZRAsync *)async operationDidSucceedWithResult:(id)result userInfo:(id)userInfo
{
  ZROperation *operation = userInfo;
  [operation didSucceedWithResult:result];  
}

- (void)async:(ZRAsync *)async operationDidFailWithError:(NSError *)error userInfo:(id)userInfo
{
  ZROperation *operation = userInfo;
  [operation didFailWithError:error];  
}

@end
