//
//  ZRModel.h
//  AsyncDispatch
//
//  Created by ampatspell on 7/23/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "ZRModelState.h"
#import "ZROperation.h"
#import "ZRRouteKind.h"
#import "ZRQueryKind.h"


@protocol ZRModelDelegate;
@class ZRModelOperation, ZRModelConfiguration;

@interface ZRModel : NSObject {
@private
  NSString *key;
  NSError *error;
  id<ZRModelDelegate> modelDelegate;
  ZRModelState state;
  BOOL disableDirtyRegistration;
}

// Configuration
+ (void)prepare;
+ (ZRModelConfiguration *)modelConfiguration;

+ (void)hasProperty:(NSString *)propertyName;
+ (void)hasProperty:(NSString *)propertyName withServerName:(NSString *)serverName;

+ (void)supports:(ZRRouteKind)kind withPath:(NSString *)path;

+ (void)query:(ZRQueryKind)kind withPath:(NSString *)path arrayClass:(Class)arrayClass;

// Init
- (id)initWithKey:(NSString *)key state:(ZRModelState)state delegate:(id<ZRModelDelegate>)delegate; // TODO: private

// State
@property(nonatomic, readwrite) ZRModelState state;
@property(nonatomic, readonly, getter=isBusy) BOOL busy;
@property(nonatomic, readonly, getter=isDestroyed) BOOL destroyed;
@property(nonatomic, readonly, getter=isCreated) BOOL created;
@property(nonatomic, readonly, getter=isDirty) BOOL dirty;
- (BOOL)hasState:(int)flag;

@property(nonatomic, readwrite, getter=isDisableDirtyRegistration) BOOL disableDirtyRegistration;
- (void)markAsDirty;

// Base properties
@property(nonatomic, readwrite, copy) NSString *key;
@property(nonatomic, readwrite, retain) NSError *error;

// Async invocations
- (void)commitWithTarget:(id)target success:(SEL)success failed:(SEL)failed;
- (void)commit;

- (void)destroyWithTarget:(id)target success:(SEL)success failed:(SEL)failed;
- (void)destroy;

- (void)fetchWithTarget:(id)target success:(SEL)success failed:(SEL)failed;
- (void)fetch;

- (void)reloadWithTarget:(id)target success:(SEL)success failed:(SEL)failed;
- (void)reload;

// Private stuff
- (void)modelOperation:(ZRModelOperation *)operation didSucceedWithResult:(NSDictionary *)result; // TODO: private
- (void)modelOperation:(ZRModelOperation *)operation didFailWithError:(NSError *)error; // TODO: private
- (void)setModelPropertiesFromDictionary:(NSDictionary *)dictionary;

@end


@interface ZRModel (ZRModelAbstract)
- (BOOL)isResultSuccessful:(NSDictionary *)result error:(NSError **)error;
- (NSDictionary *)dataDictionaryFromResult:(id)result;
@end


@interface ZRModel (ZRModelBlankImplemetations)
- (void)didLoad;
- (void)didCreate;
- (void)didUpdate;
- (void)didDestroy;

- (void)loadDidFail;
- (void)createDidFail;
- (void)updateDidFail;
- (void)destroyDidFail;
@end


@protocol ZRModelDelegate // TODO: private
- (void)performOperation:(ZROperation *)operation;
- (BOOL)retryOperation:(ZROperation *)operation withError:(NSError *)error;
@end
