//
//  ZRModelArray.h
//  AsyncDispatch
//
//  Created by ampatspell on 7/25/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "ZRModelState.h"
#import "ZRModel.h"


@class ZROperation, ZRModelArrayOperation, ZRQuery;
@protocol ZRModelArrayDelegate;

@interface ZRModelArray : NSObject <NSFastEnumeration> {
@private
  Class modelClass;
  ZRQuery *query;
  NSDictionary *queryParameters;
  id<ZRModelArrayDelegate> arrayDelegate;
  ZRModelState state;
  NSError *error;
  NSMutableArray *models;
}

- (id)initWithModelClass:(Class)modelClass
                   query:(ZRQuery *)query 
              parameters:(NSDictionary *)parameters
                   state:(ZRModelState)state 
                delegate:(id<ZRModelArrayDelegate>)delegate;


@property(nonatomic, readwrite, retain) NSError *error;

// State
@property(nonatomic, readonly) ZRModelState state;
@property(nonatomic, readonly, getter=isBusy) BOOL busy;
- (BOOL)hasState:(NSUInteger)flag;


// Fetching
- (void)fetchWithTarget:(id)target success:(SEL)success failed:(SEL)failed;
- (void)fetch;

- (void)reloadWithTarget:(id)target success:(SEL)success failed:(SEL)failed;
- (void)reload;


// Models array KVO support
- (NSUInteger)countOfModels;
- (id)objectInModelsAtIndex:(NSUInteger)index;
- (NSArray *)modelsAtIndexes:(NSIndexSet *)indexes;
- (void)insertObject:(id)object inModelsAtIndex:(NSUInteger)index;
- (void)removeObjectFromModelsAtIndex:(NSUInteger)index;
- (void)replaceObjectsAtIndexes:(NSIndexSet *)indexes withModels:(NSArray *)objects;
- (void)insertModels:(NSArray *)models atIndexes:(NSIndexSet *)indexes;
- (void)removeModelsAtIndexes:(NSIndexSet *)indexes;

- (void)addModel:(ZRModel *)model;
- (void)removeModel:(ZRModel *)model;
- (void)removeAllModels;

- (id)modelForClass:(Class)modelClass fromDictionary:(NSDictionary *)dictionary;

// Private stuff
- (void)modelArrayOperation:(ZRModelArrayOperation *)operation didSucceedWithResult:(id)result; // TODO: private
- (void)modelArrayOperation:(ZRModelArrayOperation *)operation didFailWithError:(NSError *)error; // TODO: private

@end


@interface ZRModelArray (ZRModelArrayAbstract)
- (void)willReload;
- (BOOL)isLoadingFinishedForOperation:(ZRModelArrayOperation *)operation withResultModels:(NSArray *)models;
@end

@interface ZRModel (ZRModelArrayAbstract)
+ (NSDictionary *)parametersForModelArray:(id)array;
+ (BOOL)isResultSuccessful:(id)result error:(NSError **)error forModelArray:(id)array;
+ (NSArray *)dataArrayFromResult:(id)result;
@end


@protocol ZRModelArrayDelegate
- (void)performOperation:(ZROperation *)operation;
- (BOOL)retryOperation:(ZROperation *)operation withError:(NSError *)error;
- (id)createdModelForClass:(Class)modelClass fromDictionary:(NSDictionary *)dictionary;
@end
