//
//  ZRModelConfiguration.m
//  AsyncDispatch
//
//  Created by ampatspell on 7/24/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "ZRModelConfiguration.h"
#import "ZRModelProperty.h"
#import "ZRRoute.h"
#import "ZRQuery.h"


@implementation ZRModelConfiguration
@synthesize properties, superclassConfiguration;

- (id)initWithModelClass:(Class)aModelClass superclassConfiguration:(ZRModelConfiguration *)aSuperclassConfiguration
{
  if(self = [super init]) {
    modelClass = aModelClass;
    superclassConfiguration = [aSuperclassConfiguration retain];
    properties = [[NSMutableArray alloc] init];
    propertiesForNames = [[NSMutableDictionary alloc] init];
    routesForKinds = [[NSMutableDictionary alloc] init];
    queriesForKinds = [[NSMutableDictionary alloc] init];
  }
  return self;
}

- (void)dealloc
{
  [superclassConfiguration release], superclassConfiguration = nil;
  [properties release], properties = nil;
  [propertiesForNames release], propertiesForNames = nil;
  [routesForKinds release], routesForKinds = nil;
  [queriesForKinds release], queriesForKinds = nil;
  [super dealloc];
}

#pragma mark -

- (void)addProperty:(ZRModelProperty *)settings
{
  [properties addObject:settings];
  [propertiesForNames setObject:settings forKey:settings.name];
}

- (ZRModelProperty *)propertyForName:(NSString *)name
{
  ZRModelProperty *property = [propertiesForNames objectForKey:name];
  if(!property && superclassConfiguration) {
    property = [superclassConfiguration propertyForName:name];
  }
  
  return property;
}

#pragma mark -

- (void)addRoute:(ZRRoute *)route
{
  [routesForKinds setObject:route forKey:route.kind];
}

- (ZRRoute *)routeForKind:(ZRRouteKind)kind
{
  ZRRoute *route = [routesForKinds objectForKey:kind];
  if(!route && superclassConfiguration) {
    route = [superclassConfiguration routeForKind:kind];
  }
  
  return route;
}

#pragma mark -

- (void)addQuery:(ZRQuery *)query
{
  [queriesForKinds setObject:query forKey:query.kind];
}

- (ZRQuery *)queryForKind:(ZRQueryKind)kind
{
  ZRQuery *query = [queriesForKinds objectForKey:kind];
  if(!query && superclassConfiguration) {
    query = [superclassConfiguration queryForKind:kind];
  }
  
  return query;
}

@end
